package OpenReader;

import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import java.io.*;
import javax.microedition.rms.*; //RecordStorage
import javax.xml.parsers.*;
import org.xml.sax.*;

import OpenReader.TEI_MEK_verse_parser;
import com.tinyline.util.GZIPInputStream;

public class OpenReader extends MIDlet
        implements CommandListener, Runnable {
    
    private Display display;
    private Form mainForm;
    private List Catalog;
    private Form progressForm;
    private Gauge progressGauge;
    private Thread thread;
    private Alert alert;
    private Command gCMD;
    private RecordStore bookstore;
    private TEI_MEK_verse_parser xmlhand;
    
    private final static Command CMD_CANCEL = new Command("Cancel", Command.CANCEL, 1);
    
    private final static Command CMD_LOADRES =
            new Command("Sg", Command.ITEM, 1);
    private final static Command CMD_DOWNL =
            new Command("Letlts", Command.ITEM, 1);
    private final static Command CMD_RESTORE =
            new Command("Visszatlts", Command.ITEM, 1);
    private final static Command CMD_EXIT =
            new Command("Bezr", Command.EXIT, 1);
    private final static Command CMD_SELECT =
            new Command("Megnyit", Command.ITEM, 1);
    
    public void processCMD() {
        clearscreen();
        if (gCMD == CMD_LOADRES) try{
            LoadRes();
            display.setCurrent(mainForm);
        }catch (Exception e) {
            final String msg = e.getMessage();
            alert.setString("Hibazenet a sg betltsekor: "+msg+"  "+e.toString());
            display.setCurrent(alert, mainForm);
        } else
        if (gCMD == CMD_DOWNL) try{
            DownloadText();
            display.setCurrent(mainForm);
        }catch (Exception e) {
            final String msg = e.getMessage();
            alert.setString("Hibazenet a dokumentum letltsekor: "+msg+"  "+e.toString());
            display.setCurrent(alert, mainForm);
        }else
        if (gCMD == CMD_SELECT) try{
             ViewChapter();                    
             display.setCurrent(mainForm);
        }catch (Exception e) {
             final String msg = e.getMessage();
             alert.setString("Hibazenet a fejezet betltsekor: "+msg+"  "+e.toString());
             display.setCurrent(alert, mainForm);
        } else try{
             Catalog=new List("Fejezetek",Choice.IMPLICIT);
             Catalog.addCommand(CMD_SELECT);
             Catalog.setCommandListener(this);
             LoadFromStore("Booklist",0);
             //LoadWithKXML("Booklist",0);
             display.setCurrent(Catalog);
        }catch (Exception e) {
             final String msg = e.getMessage();
             alert.setString("Hibazenet a fejezetcmek betltsekor: "+msg+"  "+e.toString());
             display.setCurrent(alert, Catalog);
        }
    }
    
    public void run() {
        processCMD();
    }
    
    private void openStore(String sname) {
        try {
            bookstore = RecordStore.openRecordStore(sname, true);
        } catch (RecordStoreException ex) {
            alert.setString("Nem lehet megnyitni a dokumentumtrolt! ("+ex.toString()+")");
            display.setCurrent(alert);
            bookstore = null;
        }
    }
    
    protected void Save2Store(byte[] svalue,String sname) {
        openStore(sname);
        if (bookstore != null) {
            try {
                mainForm.append("Mentett szveg hossza:"+Integer.toString(svalue.length)+" karakter");
                bookstore.setRecord(1, svalue, 0, svalue.length);
            } catch (InvalidRecordIDException ridex) {
                try {
                    int rec = bookstore.addRecord(svalue, 0, svalue.length);
                } catch (RecordStoreException ex) {
                    alert.setString("Dokumentum hozzadsi hiba! ("+ex.toString()+")");
                    display.setCurrent(alert);
                }
            } catch (RecordStoreException ex) {
                alert.setString("Dokumentum mentsi hiba! ("+ex.toString()+")");
                display.setCurrent(alert);
            }
            closeStore();
        }
    }
    
    protected void closeStore() {
        if (bookstore != null) {
            try {
                bookstore.closeRecordStore();
                bookstore = null;
            } catch (RecordStoreException ex) {
                alert.setString("Nem zrhat a dokumentumtrol! ("+ex.toString()+")");
                display.setCurrent(alert);
            }
        }
    }
    
    protected void DownloadText() throws IOException {
        byte[] data=null;
        int len=0;
        String type;
        String fname=getAppProperty("TextResource-2");
        mainForm.append("Letlts indtsa ("+fname+")...");
        if (fname.length()>0) {
            HttpConnection c = null;
            DataInputStream is = null;
            try {
                mainForm.append("Kapcsolat megnyitsa...");
                c = (HttpConnection)Connector.open(fname);
                mainForm.append("Kapcsolat megnyitva.");
                c.setRequestMethod(HttpConnection.GET);
                c.setRequestProperty("User-Agent", "Profile/MIDP-1.0 Configuration/CLDC-1.0");
                c.setRequestProperty("Connection", "close");
                mainForm.append("Krelem belltva.");
                int status = c.getResponseCode();
                mainForm.append("Vlasz zenet = "+Integer.toString(status));
                
                if (status != HttpConnection.HTTP_OK) {
                    throw new IOException("Hibs vlaszzenet = " + status);
                }
                
                len = (int)c.getLength();
                type = c.getType();
                if (!(type.startsWith("text/plain") || type.startsWith("application/xml") || type.startsWith("application/gz"))) {
                    throw new IOException("Nem engedlyezett dokumentum tpus (" + type +")");
                }
                mainForm.append("llomny tpusa : "+type);
                
                if (len > 0) {
                    is = c.openDataInputStream();
                    data = new byte[len];
                    is.readFully(data);
                } else {
                    throw new IOException("Tartalom adathossz hinyzik!");
                }
            } finally {
                if (is != null)
                    is.close();
                if (c != null)
                    c.close();
            };
            if (data!=null)
                Save2Store(data,"Booklist");
        } else
            mainForm.append("No fname");
    }

      /* R E A D   G Z I P   S T R E A M */
    private void LoadGZText(String sname){
        clearscreen();
        openStore(sname);
        if (bookstore != null) {
            try {
                 final DataInputStream din = new DataInputStream(new ByteArrayInputStream(bookstore.getRecord(1)));
                 final GZIPInputStream gzipis = new GZIPInputStream(din);
                 byte[] buff = new byte[1024];
                 int nch;
                 while ((nch = gzipis.read(buff, 0, buff.length)) != -1)
                   mainForm.append(new String(buff, 0, nch));
            }catch (Exception ex) {
                alert.setString("Hiba keletkezett a dokumentumtrol olvassakor! ("+ex.toString()+")");
                display.setCurrent(alert);
            } catch (OutOfMemoryError ex) {
                mainForm.append("Memria tlcsorduls! ("+ex.toString()+")");
                display.setCurrent(mainForm);
            }
            closeStore();
        }
    }

    protected void LoadWithKXML(String sname, int iproctype){
        final TEI_MEK_verse_kxml_parser parser;
        clearscreen();
        openStore(sname);
        String spos=new String();
        if (bookstore != null) {
            try {
                 final DataInputStream din = new DataInputStream(new ByteArrayInputStream(bookstore.getRecord(1)));
                 final GZIPInputStream gzipis = new GZIPInputStream(din);
                 if (iproctype==0){
                    parser = new TEI_MEK_verse_kxml_parser(mainForm);//Catalog
                 }
                 else{
                    parser = new TEI_MEK_verse_kxml_parser(mainForm);
                 }
                 spos=":form created";
                 parser.get_content(gzipis);
                 spos=":content loaded";
            }catch (Exception ex) {
             alert.setString("Hiba keletkezett a dokumentumtrol olvassakor! ("+ex.toString()+")");
             display.setCurrent(alert);
            }
            closeStore();
        }
        
    }
    
    protected void LoadFromStore(String sname, int iproctype) {
        clearscreen();
        openStore(sname);
        if (bookstore != null) {
            try {
                final DataInputStream din = new DataInputStream(new ByteArrayInputStream(bookstore.getRecord(1)));
                final GZIPInputStream gzipis = new GZIPInputStream(din);
                
                /* P A R S E */
                
                SAXParserFactory spfactory = SAXParserFactory.newInstance();
                spfactory.setValidating(false);
                
                SAXParser parser = spfactory.newSAXParser();
                if (iproctype==0) {
                    xmlhand = new TEI_MEK_verse_parser(Catalog);
                } else
                {
                    xmlhand = new TEI_MEK_verse_parser(mainForm);
                    xmlhand.idx=Catalog.getString(Catalog.getSelectedIndex()).toLowerCase().trim();
                    mainForm.append(Catalog.getString(Catalog.getSelectedIndex())+"\n\n");
                }
                xmlhand.fProcType=iproctype;
                InputSource source = new InputSource(gzipis);
      
                final String enc = source.getEncoding();
                if (enc!=null)
                  mainForm.append("encoding:"+enc+"\n");
                
                parser.parse(source, xmlhand);   // P A R S E ------------------
                
            }catch (SAXException ex) {
                if (!(ex.getMessage().equals("break"))) {
                    alert.setString("Hiba keletkezett a dokumentumtrol olvassakor! ("+ex.toString()+")");
                    display.setCurrent(alert);
                }
        }catch (Exception ex) {
            alert.setString("Hiba keletkezett a dokumentumtrol olvassakor! ("+ex.toString()+")");
            display.setCurrent(alert);
            } catch (OutOfMemoryError ex) {
                mainForm.append("Memria tlcsorduls! ("+ex.toString()+")");
                display.setCurrent(mainForm);
            }
            closeStore();
        }
    }
    
    protected void LoadRes() throws IOException {
        clearscreen();
        StringBuffer svalue=new StringBuffer();
        String fname=getAppProperty("TextResource-1");
        if (fname.length()>0) {
            InputStream istrm = getClass().getResourceAsStream(fname);
            byte[] buff = new byte[4096];
            int nch;
            while ((nch = istrm.read(buff, 0, buff.length)) != -1) {
                svalue=svalue.append(new String(buff, 0, nch));
            }
            mainForm.append(svalue.toString());
            istrm.close();
        }
    }
    protected void ViewChapter() {
        //LoadFromStore("Booklist", 1);
        LoadWithKXML("Booklist", 1);
    }
    
    protected void startApp() {
        display = Display.getDisplay(this);
        
        mainForm = new Form("Mobil knyvespolc");
        alert = new Alert("Warning");
        
        mainForm.append("Internetrl letlthet tartalom:");
        mainForm.append(getAppProperty("TextResource-2")+"\n");
        mainForm.addCommand(CMD_EXIT);
        mainForm.addCommand(CMD_LOADRES);
        mainForm.addCommand(CMD_DOWNL);
        mainForm.addCommand(CMD_RESTORE);
        mainForm.setCommandListener(this);
        display.setCurrent(mainForm);
    }
    
    private Screen genProgress(String name) {
        if (progressForm == null) {
            progressForm = new Form(name);
            progressForm.addCommand(CMD_CANCEL);
            progressForm.setCommandListener(this);
            progressGauge = new javax.microedition.lcdui.Gauge("File letltse...",false, 9, -1);
            progressForm.append(progressGauge);
        } else {
            progressGauge.setValue(0);
            progressForm.setTitle(name);
        }
        return progressForm;
    }
    
    private void clearscreen() {
        //mainForm.deleteAll(); MIDP2.0
        for(int i=mainForm.size()-1;i>-1;i--)
            mainForm.delete(i);
    }
    
    public void commandAction(Command c, Displayable d) {
        if ((c!=CMD_EXIT)&&(c!=CMD_CANCEL)) {
            gCMD=c;
            int i=0;
            //display.setCurrent(genProgress("Letlts:"));
            thread = new Thread(this);
            thread.start();
        } else
            if(c==CMD_CANCEL) {
            thread = null;
            alert.setString("Fjl letlts megszaktva.");
            display.setCurrent(alert, mainForm);
            } else {
            destroyApp(false);
            notifyDestroyed();
            }
    }
    
    /**
     * Signals the MIDlet to terminate and enter the Destroyed state.
     */
    protected void destroyApp(boolean unconditional) {
    }
    
    /**
     * Signals the MIDlet to stop and enter the Paused state.
     */
    protected void pauseApp() {
    }
}
